/* mash1_c subroutine
 * First order MASH Sigma-Delta Modulator
 * IRCOM GROUP - Author : A.Layec
 */
 
/* REVISION HISTORY :
 * $Log$
 */

#include "mod_num_lib.h"

/* mash1_c routine de calcul d'un modulateur sigma-delta du 1er ordre type MASH
 *
 * entres :
 * n   : longueur du vecteur d'entre (scalaire)
 * m   : gain du modulateur (scalaire)
 * u   : adresse de dpart du vecteur  convertir 
 * entres/sorties 
 * e   : intgral du signal d'erreur du dernier lment (scalaire)
 * z   : etat de sortie du dernier lment +m;-m (scalaire)
 * y   : adresse de dpart du vecteur de sortie (+1;-1)
 * sorties :
 * q   : adresse de dpart du vecteur de sortie de l'erreur de quantification
 */
 
void mash1_c(int *n,double *m,double *u,double *e,double *z,double *y,double *q)
{
 /*dclaration*/
 int i;
 
 for(i=0;i<(*n);i++)
 {
  /*Calcul de l'intgrale du signal d'erreur*/
  *e+=u[i]-(*z);
    
  /*Ralisation de la comparaison de signe*/
  if((*e)>0) y[i]=1;
  else y[i]=-1;
   
  /*mise en mmoire de la sortie*/
  *z = *m*y[i];
  
  /*Calcul du bruit de quantification*/
  q[i]=(*z)-(*e);
 }
 return;
}
