/*
 * awgn Scicos Additive White Gaussian Noise channel block
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 23 dcembre 2004 - IRCOM GROUP - Author : A.Layec
 */
 
/* REVISION HISTORY :
 * $Log$
 */
 
#include "scicos_block.h"

/* Cette fonction de simulation calcule un vecteur alatoire gaussien
 * par la mthode "Muler Box" et l'ajoute au vecteur complexe d'entre
 *
 * y1[i]=u1[i]+mean+sigma*sqrt(-log(rand1))*cos(2*%pi*rand2) 
 * y2[i]=u2[i]+mean+sigma*sqrt(-log(rand1))*sin(2*%pi*rand2)  
 *
 * o mean et sigma sont des paramtres et rand1 et rand2 sont
 * des variables alatoires uniformment rparties
 *
 * entre rgulire : u1[0..nu-1] vecteur des parties rlles des chantillons d'entre
 *                    u2[0..nu-1] vecteur des parties imaginaires des chantillons d'entre
 * sortie rgulire : y1[0..ny-1] vecteur des parties rlles des chantillons de sortie
 *                    y2[0..ny-1] vecteur de parties imaginaires des chantillons de sortie
 * entre vnementielle : Dates de dclenchement
 * sortie vnementeille : nant
 *
 * paramtre entier : nant
 * paramtre rel : rpar[0] : valeur du sigma
 *                  rpar[1] : valeur de la moyenne
 */

/*prototype*/
void awgn(scicos_block *block,int flag)
{
  /*dclaration des variables*/
  double *u1,*u2;
  double *y1,*y2;
  int k;
  int ny;

  /*rcupration de l'adresses des ports rguliers*/
  u1=(double *)block->inptr[0];
  u2=(double *)block->inptr[1];
  y1=(double *)block->outptr[0];
  y2=(double *)block->outptr[1];

  /*rcupre taille de sortie*/
  ny=block->outsz[0];
  
  /*Le flag 1 calcule le registre de sortie y*/
  if(flag==1)
  {
   /*Appel noiseiq_c*/
   noiseiq_c(&ny,&block->rpar[0],&block->rpar[1],&y1[0],&y2[0]);
   
   /*Appel complxa_c*/
   cmplxa_c(&ny,(k=1,&k),&u1[0],&u2[0],&y1[0],&y2[0],&y1[0],&y2[0]);
  }
}
