/* destockbit Scicos binary shift register
 * Type 2 simulation function ver 1.1 - scilab-2.6&2.7&3.0
 * 12 novembre 2003 - IRCOM GROUP - Author : A.Layec
 */
 
/* REVISION HISTORY :
 * $Log$
 */
 
#include "machine.h"
#include <stdio.h>

/* Cette fonction ralise un registre  dcalage binaire.
 * A chaque vnement prsent sur port vnementiel numro 1
 * la fonction ralise une acquistion de la valeur prsente
 * sur son port d'entre rgulier suppos etre de taille ipar[0..nu-1]*ipar[nu..2*nu-1]
 * puis dlivre un paquet de taille ipar[0..nu-1] sur le port de sortie rgulier lorsque
 * un vnement se prsente sur son port vnementiel 2.
 * A chaque vnement 2, un ET logique de ipar[0..nu-1] bits est ralis
 * et le registre est dcal de ipar[0..nu-1] bits vers la droite.
 * 
 * Entre rgulire : u[0..nu-1] vecteur des entiers cods sur Nbit*Nbpaquets
 * Sortie rgulire : y[0..nu-1] vecteur des entiers cods sur Nbit
 * Entre vnementielle : nev=1 : Instant d'acquisition du mot en entre de taille Nbit*Nbpaquet
 *                         nev=2 : Instant de dlivrance du mot en sortie de taille Nbit
 * Sortie vnementielle : nant
 * Paramtres entiers : ipar[0..nu-1] : longueur des paquets du vecteur d'entre (Nbit)
 *                      ipar[nu..2*nu-1] : Nombre de paquet
 * Mmoires : z[0..nu-1] : valeur paquet
 */
 
/*prototype*/
void destockbit(flag,nevprt,t,xd,x,nx,z,nz,tvec,ntvec,rpar,nrpar,
                ipar,nipar,inptr,insz,nin,outptr,outsz,nout)
integer *flag,*nevprt,*nx,*nz,*ntvec,*nrpar,ipar[],*nipar,insz[],*nin,outsz[],*nout;
double x[],xd[],z[],tvec[],rpar[];
double *inptr[],*outptr[],*t;
{
  /*dclaration des variables*/
  double *y;
  double *u;
  int nu,nev,i,j,k;

  /*Rcupration des adresses des ports*/
  y=(double *)outptr[0];
  u=(double *)inptr[0];
  nev=*nevprt;

  /*rcupration de la taille du port d'entre rgulier*/
  nu=insz[0];

  /* Le flag 1 rcupre le paquet  dlivrer
   * et dlivre sa valeur cc2 dans y[]
   * lorsque le port d'activation est 2 ou 3
   */
   
  if(*flag==1 && (nev==2||nev==3))
  {
   for(k=0;k<nu;k++)
   {
    /*Rcupration paquet entre*/
    i=(int)z[k];

    /*Tronque valeur de i ux ipar[k] bits de poid faible*/
    j=i&((1<<ipar[k])-1);

    /*Ralise conversion code complment  2*/
    j-= 2<<(ipar[k]-2);
    j&= (2<<(ipar[k]-1)) - 1;
    j-= 2<<(ipar[k]-2);

    /*place valeur dans y[]*/
    y[k]=j;

    /*Dcale paquet de ipar[0] vers la droite*/
    i=i>>ipar[k];

    /*Sauvegarde valeur dcale dans z[k]*/
    z[k]=i;
    /*fprintf(stderr,"z[k]_sauv_bloc2=%f\n", z[k]);*/
   }
  }

  /* Le flag 2 stocke la valeur en entre dans z[]
   * et la convertie en nombre entier non sign lorsque
   * le numro du port d'activation est 1 ou 3
   */
  else if(*flag==2 && (nev==1||nev==3))
    {
     for(k=0;k<nu;k++)
     {
      /*aquisition paquet entre*/
      j=(int)u[k];

      /*Conversion en nombre entier non sign*/
      j &= (2<<((ipar[k]*ipar[nu+k])-1)) - 1;

      /*Sauvegarde paquet entre dans z[k]*/
      z[k]=j;
     }
    }
}
