/* modpsk Scicos Mary Phase Shift Keying modulator block
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 21 dcembre 2004 - IRCOM GROUP - Author : A.Layec
 */
 
/* REVISION HISTORY :
 * $Log$
 */
 
#include "scicos_block.h"
#include "machine.h"
#include <math.h>
#include <stdio.h>
#define M_PI		3.14159265358979323846

/* Cette fonction de simulation ralise un codeur MPSK.
 * Les sorties y1 et y2 correspondent aux valeurs de I et de Q
 * codes en fonction du nombre d'tats et du numro symbole
 * prsent  l'entre u[] aux dates de dclanchements.
 * La fonction calcule :
 * I=cos(%pi*(2*u+1)/m) 
 * Q=-sin(%pi*(2*u+1)/m)
 * o u est l'entre et m le nombre d'tats.
 * 
 * entres rgulires      : u[0..insz[0]-1]  : vecteur des numros symboles
 * sorties rgulires      : y1[0..insz[0]-1] : vecteur des composantes I
 *                           y2[0..insz[0]-1] : vecteur des composantes Q
 * entre vnementielles  : instants de dclenchement.
 * sortie vnementielle   : nant
 * paramtres entiers      : ipar[0..insz[0]-1]:m vecteur des nombre d'tats
 */

/* modpskv_c routine de calcul des composantes I et Q en fonction
 * d'un numro symbole cod par tats de phase (psk)
 *
 * Entres :
 * n   : longueur des vecteurs (scalaire)
 * m   : nombre d'tat Attention - ici vecteur
 * u   : numro symbole (vecteur d'entre)
 * Sorties :
 * i_c : valeur de la composante i (vecteur de sortie 1)
 * q_c : valeur de la composante q (vecteur de sorite 2)
 *
 * Dpendances :
 * math.h
 */
void modpskv_c(int *n,int *m,double *u,double *i_c, double *q_c)
{
 /*Dclaration des variables compteurs*/
 int i;
 
 for(i=0;i<(*n);i++)
 {
  i_c[i]=cos(M_PI*(2*u[i]+1)/m[i]);
  q_c[i]=-sin(M_PI*(2*u[i]+1)/m[i]);
 }
 return;
}

/*prototype*/
void modpsk(scicos_block *block,int flag)
{
 /*Dclaration des variables*/
 double *y1,*y2;
 double *u;
 int i;

 /*Rcupration des adresses des ports rguliers*/
 y1=(double *)block->outptr[0];
 y2=(double *)block->outptr[1];
 u=(double *)block->inptr[0];

 /*
 Le flag 1 calcule la valeur de I et de Q en fonction
 du numro symbole u[] et du nombre du nombre d'tat m
 */
 if(flag==1||flag==6)
 {
  /*Appel routine modpsk*/
  modpskv_c(&block->insz[0],&block->ipar[0],&u[0],&y1[0],&y2[0]);
 }
}
