/* modqam Scicos QAM modulator block
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 5 janvier 2004 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */
 
#include "machine.h"
#include <stdio.h>
#include "scicos_block.h"

/* Cette fonction de simulation ralise un codage type MQAM.
 * Le numro symbole doit etre dfinis par un nombre entier non sign dont
 * la valeur est code par un nombre de bits pair (4 bits, 6 bits, 8 bits,...).
 * Les slecteurs ng (slecteur bits de poid faibles) et nd (slecteurs bits de poid fort)
 * viennent tronquer la valeur du port d'entre.
 * On associe les valeurs entires des bits de poid faibles tronques aux valeurs de l'axe
 * I, et on associe les valeurs entires des bits de poids forts tronques et dcales
 * aux valeurs de l'axe Q.
 *
 * Entre rgulire :   u[0..insz[0]-1] : vecteur des numros symboles
 * Sorties rgulires : y1[0..insz[0]-1]: vecteur des signaux I
 *                      y2[0..insz[0]-1]: vecteur des signaux Q
 * Entre vnementielle : instants de dclenchement
 * Sortie vnementielle : nant
 * Paramtres entiers : ipar[0..insz[0]-1] : vecteur des nombres de bits.
 * 
 * nb : ne fonctionne que pour des modulations avec un nombre de valeurs
 *      identiques pour chaque axes I et Q(16QAM, 64QAM, 256QAM, ...)
 */

/* modqamv_c routine de calcul d'un modulataur mQAM
 *
 * Entres :
 * n :longueur du vecteur d'entre
 * m :longueurs des mots binaires en nombre de bits (vecteur)
 * u :adresse de dpart du vecteur du symbole en entre
 *
 * Sortie :
 * i_c : adresse de dpart du vecteur de la composante I
 * q_c : adresse de dpart du vecteur de la composante Q
 *
 * Dpendances:
 */
void modqamv_c(int *n,int *m,double *u,double *i_c,double *q_c)
{
 /*dclaration des variables compteurs*/
 int i,j;
 int ng,nd;
 
 for(j=0;j<(*n);j++)
 {
  /*rcupration du nombre de bits associ  chaque axes*/
  /*n = block->ipar[j];*/

  /*Calcul des slecteurs binaires (c'est maladroit!!)*/
  nd=(1<<(m[j])/2)-1;
  ng=(1<<(m[j]))-1-nd;

  /*rcupration de la valeur du port d'entre*/
  i=(int)u[j];

  /*Calcul de la valeur de I et de Q*/
  i_c[j]=((i&nd)*2)-nd;
  q_c[j]=(((i&ng)>>((m[j])/2))*2)-nd;
 }
 return;
}

/*prototype*/
void modqam(scicos_block *block,int flag)
{
  /*dclaration des variables*/
  double *y1,*y2;
  double *u;
  int i,j,n,nu;
  int nd,ng;

  /*Rcupration des adresses des ports rguliers*/
  y1=(double *)block->outptr[0];
  y2=(double *)block->outptr[1];
  u=(double *)block->inptr[0];

  /*rcupration du nombre de symbole en entre*/
  nu=block->insz[0];
  
  /* Le flag 1 calcule les valeurs de I et de Q
   * suivant le numro symbole u et suivant le
   * nombre de bits associ  chaque axes
   */
   
  if(flag==1||flag==6)
  {
   /*Appel modqamv_c*/
   modqamv_c(&nu,&block->ipar[0],&u[0],&y1[0],&y2[0]);   
  }
}
