/* vco Scicos VCO with frequency divider Event block
 * Type 4 simulation function ver 1.1 - scilab-3.0
 * 1 decembre 2003 - IRCOM GROUP - Author : A.Layec
 * 10 janvier 2005 : rajout option du rapport de division dynamique
 *                   passage type 4
 *                   remplace otutes les anciennes routines vco
 */
 
/* REVISION HISTORY :
 * $Log$
 */
 
#include "machine.h"
#include <math.h>
#include <stdio.h>
#include "scicos_block.h"

/* Calcul la forme temporelle de sortie d'un VCO et ses instants de transitions
 * produit un vnement en sortie toutes les N fois (N:rapport de division)
 *
 * entres rgulires : u1[0] : u3(t) tension de commande
 *                     u2[0] : valeur du rapport de division
 *
 * sorties rgulires : y[0] : ys(t)=cos(wot+phi(t)) 
 *                 avec phi(t)=integral(Kv*u3(t))dt pour le cas linaire 
 *                phi(t)=intgral(gain*tanh(coef*u3(t)))dt pour le cas non-linaire
 *
 * entres vnementielles : Tstep pas de rsolution
 *
 * sorties vnementielles : evout[0] : Instant de passage aux valeurs rpar[3]*(ipar[0]+u1[0])
 * 
 * paramtres rels :   rpar[0] : pas d'intgration
 *                      rpar[1] : wo pulsation libre de l'oscillateur
 *                      rpar[2] : kv sensibilit
 *                      rpar[3] : instant de transition
 *                    si ipar[1]==1 :
 *                      rpar[4] : gain de la fonction non-linaire
 *                      rpar[5] : coeffficient de la fonction non-linaire
 *                      rpar[6] : moyenne du bruit sur la commande
 *                      rpar[7] : variance du bruit sur la commande
 * 
 * paramtres entiers : ipar[0] : ralise une dtection dynamique
 *                               0 : passage aux valeurs rpar[1]
 *                                1 : passage aux valeurs rpar[1]*u2[0]
 *                      ipar[1] : ralise la fonction non-linaire en entre du VCO
 *                                0 : linaire
 *                                1 : alpha*tanh(beta*u1[0])
 *                      ipar[2] : ralise un deuxime diviseur de frquence  rapport entier
 *                                0 : pas de sencond diviseur
 *                              sinon valeur du 2eme rapport
 *                      ipar[3] : type de bruit
 *                                0 : pas de bruit
 *                                1 : bruit normal sur la commande d'entre
 * 
 * tat discret : z[0] : valeur de la phase instantanne
 *               z[1] : valeur de l'entre  l'tat prcdent
 *                z[2] : valeur de l'angle instantanne
 *               z[3] : valeur de 'angle  l'tat prcdent
 *                z[4] : compteur incremental de l'angle (ex: ..+2*%pi*N[k-1]+2*%pi*N[k]+..)
 *                z[5] : valeur de la distance  parcourir jusqu' la prochaine transition (ex : 2*%pi*N)
 *                z[6] : compteur incremental de l'angle du 2eme diviseur (ex: ..+2*%pi*N[k-1]+2*%pi*N[k]+..)
 */
 
/*prototype*/
void vco(scicos_block *block,int flag)
{
 /*dclaration des variables*/
 double *y;
 double *u1;
 double *u2;

 double step; /*pas d'intgration*/
 double wo; /*pulsation libre*/
 double kv; /*sensibilit*/
 double v; /*valeur de passage*/
 double t; /*valeur du temps de simulation*/
 double f_u; /*valeur de l'entre(avec sa fonction)*/
 double alpha; /*gain de la fonc non-linaire*/
 double beta; /*coef de la func non-linaire*/
 
 /*Rcupration des adresses des ports rguliers*/
 y=(double *)block->outptr[0];
 u1=(double *)block->inptr[0];
 u2=(double *)block->inptr[1];
 step=block->rpar[0];
 wo=block->rpar[1];
 kv=block->rpar[2];
 v=block->rpar[3];
   
 /* Le flag 1 calcule l'intgrale discrte (trapze) de la tension d'entre
  * calcul de l'angle instantanne theta(t)=(wot+phi(t))
  * Place le cos(theta(t)) dans y[]
  */
 if(flag==1)
 {
  /*Calcul de la fonction d'entre*/
  if (block->ipar[1]!=0)
  { 
   alpha=block->rpar[4];
   beta=block->rpar[5];
   f_u=alpha*tanh(beta*u1[0]);
  }
  else f_u=u1[0];
  /*multiplication par sensibilit linaire*/
  f_u=kv*f_u; 
  
  if (block->ipar[3]==1) /*Test si bruit*/
  {
   /*dclaration*/
   double phi_n;
   int i,k;
   
   /*Appel noiseblk_c*/
   noiseblk_c((i=1,&i),(k=1,&k),&block->rpar[7],&block->rpar[6],&phi_n);
   
   /*rajout bruit*/
   f_u=f_u+(phi_n);
  }
  /*Calcul de la phase instantanne : intgrale de la valeur d'entre par la mthode des trapzes*/
  block->z[0]=(step/2)*(f_u+block->z[1])+block->z[0];
  /*Mise en mmoire de f_u*/
  block->z[1]=f_u;
  /*Calcul de l'angle wo*t+phi(t)*/
  t=get_scicos_time(); /*rcupration de la valeur du temps*/
  block->z[2]=t*wo+block->z[0];
 
  /*Calcul du cos(wo*t+phi(t))*/
  y[0]=cos(block->z[2]);
 }

 /* Le flag 3 dtermine l'instant de passage de theta(t)  rpar[3]*ipar[0]->z[5]
  * par une mthode d'extrapolation et place le rsultat dans tvec[0]
  */
  
 else if(flag==3)
 {
  if (block->ipar[0]==1)
  {
   /*dclaration*/ 
   double *u2;
   /*rcupration adresse*/
   u2=(double *)block->inptr[1];
   /*fprintf(stderr,"U2[0]=%f\n",u2[0]);*/
   block->z[5]=v*u2[0];
  }
  else block->z[5]=v;
  /*block->z[5]=v*block->ipar[0];*/
    
  /*Teste si la valeur de transistion est dpasse*/
  if((block->z[2]-block->z[4])>=block->z[5])
  {
   /*Calcul de l'instant de transition*/
   block->evout[0]=(((block->z[4]+block->z[5])-block->z[3])*step)/(block->z[2]-block->z[3]);
   
   /*Incrment valeur z[4]*/
   block->z[4]=block->z[4]+block->z[5];
  }
  
  /*Test si il existe un deuxime port de sortie evnenemtiel*/
  /*fprintf(stderr,"nevout=%d\n",block->nevout);*/
  if(block->ipar[2]!=0) 
  {
   if((block->z[2]-block->z[6])>=v*block->ipar[2])
   {
    /*Calcul de l'instant de transition*/
    block->evout[1]=(((block->z[6]+v*block->ipar[2])-block->z[3])*step)/(block->z[2]-block->z[3]);
    /*Incrment valeur z[6]*/
    block->z[6]=block->z[6]+v*block->ipar[2];
   }
  }
 }

 /*Le flag 2 mmorise la valeur de thetha_k-1 dans z[3]*/
 else if(flag==2)
 {
  /*Mise en mmoire de*/
  block->z[3]=block->z[2];
 }
}
